Type: Package
Package: cardinalR
Title: Collection of Data Structures
Version: 1.0.6
Authors@R: c(
    person("Jayani P.", "Gamage", , "jayanilakshika76@gmail.com", role = c("aut", "cre"),
           comment = c(ORCID = "0000-0002-6265-6481")),
    person("Dianne", "Cook", , "dicook@monash.edu", role = "aut",
           comment = c(ORCID = "0000-0002-3813-7155")),
    person("Paul", "Harrison", , "paul.harrison@monash.edu", role = "aut",
           comment = c(ORCID = "0000-0002-3980-268X")),
    person("Michael", "Lydeamore", , "michael.lydeamore@monash.edu", role = "aut",
           comment = c(ORCID = "0000-0001-6515-827X")),
    person("Thiyanga S.", "Talagala", , "ttalagala@sjp.ac.lk", role = "aut",
           comment = c(ORCID = "0000-0002-0656-9789"))
  )
Description: A collection of functions to generate a large variety of
    structures in high dimensions. These data structures are useful for
    testing, validating, and improving algorithms used in dimensionality
    reduction, clustering, machine learning, and visualization.
License: MIT + file LICENSE
URL: https://jayanilakshika.github.io/cardinalR/
BugReports: https://github.com/JayaniLakshika/cardinalR/issues
Depends: R (>= 4.1.0)
Imports: cli, dplyr, geozoo, gtools, MASS, mvtnorm, purrr, stats,
        tibble, tidyr
Suggests: knitr, langevitour, rmarkdown, testthat (>= 3.0.0)
VignetteBuilder: knitr
Config/testthat/edition: 3
Encoding: UTF-8
Language: en-GB
LazyData: true
RoxygenNote: 7.3.3
NeedsCompilation: no
Packaged: 2025-12-18 03:04:06 UTC; jpiy0001
Author: Jayani P. Gamage [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-6265-6481>),
  Dianne Cook [aut] (ORCID: <https://orcid.org/0000-0002-3813-7155>),
  Paul Harrison [aut] (ORCID: <https://orcid.org/0000-0002-3980-268X>),
  Michael Lydeamore [aut] (ORCID:
    <https://orcid.org/0000-0001-6515-827X>),
  Thiyanga S. Talagala [aut] (ORCID:
    <https://orcid.org/0000-0002-0656-9789>)
Maintainer: Jayani P. Gamage <jayanilakshika76@gmail.com>
Repository: CRAN
Date/Publication: 2025-12-18 08:10:17 UTC
Built: R 4.6.0; ; 2026-04-29 13:23:56 UTC; unix
