# Generated by roxygen2: do not edit by hand

export(gen_bkgnoise)
export(gen_circle)
export(gen_clusteredspheres)
export(gen_clustloc)
export(gen_cone)
export(gen_conicspiral)
export(gen_crescent)
export(gen_cubic)
export(gen_curvybranches)
export(gen_curvycycle)
export(gen_curvycylinder)
export(gen_expbranches)
export(gen_gaussian)
export(gen_gridcube)
export(gen_gridedsphere)
export(gen_helicalspiral)
export(gen_hemisphere)
export(gen_hole)
export(gen_hollowsphere)
export(gen_linearbranches)
export(gen_longlinear)
export(gen_mobius)
export(gen_multicluster)
export(gen_noisedims)
export(gen_nonlinear)
export(gen_nproduct)
export(gen_nsum)
export(gen_orgcurvybranches)
export(gen_orglinearbranches)
export(gen_pyrfrac)
export(gen_pyrrect)
export(gen_pyrstar)
export(gen_pyrtri)
export(gen_quadratic)
export(gen_rotation)
export(gen_scurve)
export(gen_scurvehole)
export(gen_sphericalspiral)
export(gen_swissroll)
export(gen_trefoil3d)
export(gen_trefoil4d)
export(gen_unifcube)
export(gen_unifcubehole)
export(gen_unifsphere)
export(gen_wavydims1)
export(gen_wavydims2)
export(gen_wavydims3)
export(make_chain_circles)
export(make_chain_curvycycle)
export(make_curvygau)
export(make_gaucircles)
export(make_gaucurvycycle)
export(make_klink_circles)
export(make_klink_curvycycle)
export(make_mobiusgau)
export(make_multigau)
export(make_onegrid)
export(make_shape_para)
export(make_twogrid_overlap)
export(make_twogrid_shift)
export(normalize_data)
export(randomize_rows)
export(relocate_clusters)
import(dplyr)
import(purrr)
import(tibble)
