---
title: "Mobius data"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Mobius data}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

```{r setup}
library(cardinalR)
library(langevitour)
```

```{r}
mobius_data <- mobius_5d(n = 300, num_noise = 2, 
          min_n = -0.05, max_n = 0.05)

colnames(mobius_data) <- paste0("x", 1:NCOL(mobius_data))

langevitour(mobius_data, pointSize = 2)
```

```{r}
mobius_clust_data <- mobius_clust(n = 500, 
num_noise = 2, min_n = -0.05, max_n = 0.05)

colnames(mobius_clust_data) <- paste0("x", 1:NCOL(mobius_clust_data))

langevitour(mobius_clust_data, pointSize = 2)
```
