---
title: "S-curve data"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{S-curve data}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

```{r setup}
library(cardinalR)
library(langevitour)
```

```{r}
scurve_data <- scurve(n = 500, num_noise = 2, 
        min_n = -0.05, max_n = 0.05)

colnames(scurve_data) <- paste0("x", 1:NCOL(scurve_data))

langevitour(scurve_data, pointSize = 2)
```

```{r}
scurve_hole_data <- scurve_hole(n = 500, num_noise = 2, 
             min_n = -0.05, max_n = 0.05)

colnames(scurve_hole_data) <- paste0("x", 1:NCOL(scurve_hole_data))

langevitour(scurve_hole_data, pointSize = 2)
```

```{r}
two_scurve_data <- two_scurves(n = 500, num_noise = 2, 
                        min_n = -0.05, max_n = 0.05)

colnames(two_scurve_data) <- paste0("x", 1:NCOL(two_scurve_data))

langevitour(two_scurve_data, pointSize = 2)
```

```{r}
mirror_scurve_data <- mirror_scurves(n = 500, num_noise = 2, 
                           min_n = -0.05, max_n = 0.05)

colnames(mirror_scurve_data) <- paste0("x", 1:NCOL(mirror_scurve_data))

langevitour(mirror_scurve_data, pointSize = 2)
```

```{r}
two_scurveh_data <- two_scurve_hole(n = 500, num_noise = 2, 
                            min_n = -0.05, max_n = 0.05)
two_scurveh_data$n

colnames(two_scurveh_data$df) <- paste0("x", 1:NCOL(two_scurveh_data$df))

langevitour(two_scurveh_data$df, pointSize = 2)
```
