---
title: "Sphere data"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Sphere data}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

```{r setup}
library(cardinalR)
library(langevitour)
```

```{r}
sphere_data <- sphere(radius = 1, resolution = 20, num_noise = 2, 
                      min_n = -0.05, max_n = 0.05)

colnames(sphere_data) <- paste0("x", 1:NCOL(sphere_data))

langevitour(sphere_data, pointSize = 2)
```

```{r}
diff_sphere_data <- diff_sphere(n = 650, 
                                                num_noise = 2, 
                                                min_n = -0.05, max_n = 0.05)

colnames(diff_sphere_data) <- paste0("x", 1:NCOL(diff_sphere_data))

langevitour(diff_sphere_data, pointSize = 2)
```

