---
title: "Branching data"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Branching data}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

```{r setup}
library(cardinalR)
library(langevitour)
```

```{r}
curvy_tree_data <- curvy_tree(n = 720, 
num_noise = 2, min_n = -0.05, max_n = 0.05)

colnames(curvy_tree_data) <- paste0("x", 1:NCOL(curvy_tree_data))

langevitour(curvy_tree_data, pointSize = 2)
```

```{r}
tree_data <- tree(n = 490, num_noise = 2, 
min_n = -0.05, max_n = 0.05)

colnames(tree_data) <- paste0("x", 1:NCOL(tree_data))

langevitour(tree_data, pointSize = 2)
```

```{r}
seven_branch_data <- seven_branch(n = 490, 
num_noise = 2, min_n = -0.05, max_n = 0.05)

colnames(seven_branch_data) <- paste0("x", 1:NCOL(seven_branch_data))

langevitour(seven_branch_data, pointSize = 2)
```

```{r}
four_branch_data <- four_branch(n = 880, 
num_noise = 2, min_n = -0.05, max_n = 0.05)

colnames(four_branch_data) <- paste0("x", 1:NCOL(four_branch_data))

langevitour(four_branch_data, pointSize = 2)
```

```{r}
eight_branch_data <- eight_branch(n = 600, 
num_noise = 2, min_n = -0.05, max_n = 0.05)

colnames(eight_branch_data) <- paste0("x", 1:NCOL(eight_branch_data))

langevitour(eight_branch_data, pointSize = 2)
```

```{r}
curvy_clust_data <- curvy_branch_clust(n = 500, 
clust_vec = c(200, 150, 150), num_noise = 2, 
min_n = -0.05, max_n = 0.05)

colnames(curvy_clust_data) <- paste0("x", 1:NCOL(curvy_clust_data))

langevitour(curvy_clust_data, pointSize = 2)
```

```{r}
curvy_branchingbkg_data <- curvy_branch_clust_bkg(n = 500, num_noise = 2, min_n = -0.05, max_n = 0.05)

colnames(curvy_branchingbkg_data) <- paste0("x", 1:NCOL(curvy_branchingbkg_data))

langevitour(curvy_branchingbkg_data, pointSize = 2)
```

```{r}
curvy_branching_data <- curvy_branch(n = 500, 
num_noise = 2, min_n = -0.05, max_n = 0.05)

colnames(curvy_branching_data) <- paste0("x", 1:NCOL(curvy_branching_data))

langevitour(curvy_branching_data, pointSize = 2)
```
