---
title: "Circular data"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Circular data}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

```{r setup}
library(cardinalR)
library(langevitour)
```

```{r}
three_circ_data <- three_circulars(n = 300, num_noise = 2, min_n = -0.05, max_n = 0.05)

colnames(three_circ_data) <- paste0("x", 1:NCOL(three_circ_data))

langevitour(three_circ_data, pointSize = 2)
```

```{r}
cell_cyc_data <- cell_cycle(n = 300, num_noise = 2, min_n = -0.05, max_n = 0.05)

colnames(cell_cyc_data) <- paste0("x", 1:NCOL(cell_cyc_data))

langevitour(cell_cyc_data, pointSize = 2)
```

```{r}
curvy_cyc_data <- curvy_cycle(n = 300, num_noise = 2, min_n = -0.05, max_n = 0.05)

colnames(curvy_cyc_data) <- paste0("x", 1:NCOL(curvy_cyc_data))

langevitour(curvy_cyc_data, pointSize = 2)
```

```{r}
linkdata <- two_circulars(n = 200, num_noise = 2, min_n = -0.05, max_n = 0.05)

colnames(linkdata) <- paste0("x", 1:NCOL(linkdata))

langevitour(linkdata, pointSize = 2)
```

