---
title: "Grid data"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Grid data}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

```{r setup}
library(cardinalR)
library(langevitour)
```

```{r}
grd_data <- one_grid(nx = 10, ny = 10, num_noise = 2, min_n = -0.05, max_n = 0.05)

colnames(grd_data) <- paste0("x", 1:NCOL(grd_data))

langevitour(grd_data, pointSize = 2)
```

```{r}
one_grdbkg_data <- one_grid_bkg(n_value = 10, num_noise = 2, min_n = -0.05, max_n = 0.05)
one_grdbkg_data$n

colnames(one_grdbkg_data$df) <- paste0("x", 1:NCOL(one_grdbkg_data$df))

langevitour(one_grdbkg_data$df, pointSize = 2)
```

```{r}
two_grd_data <- two_grid(n_value = 19, num_noise = 2, min_n = -0.05, max_n = 0.05)
two_grd_data$n

colnames(two_grd_data$df) <- paste0("x", 1:NCOL(two_grd_data$df))

langevitour(two_grd_data$df, pointSize = 2)
```

```{r}
two_grd_data <- two_grid_comb_bkg(n_value = 15, num_noise = 2, min_n = -0.05, max_n = 0.05)
two_grd_data$n

colnames(two_grd_data$df) <- paste0("x", 1:NCOL(two_grd_data$df))

langevitour(two_grd_data$df, pointSize = 2)
```

```{r}
one_grddiff_data <- two_grid_comb(n = 512, num_noise = 2, min_n = -0.05, max_n = 0.05)
one_grddiff_data$n

colnames(one_grddiff_data$df) <- paste0("x", 1:NCOL(one_grddiff_data$df))

langevitour(one_grddiff_data$df, pointSize = 2)
```

```{r}
three_grd_data <- three_grid(n_value = 19, num_noise = 2, min_n = -0.05, max_n = 0.05)
three_grd_data$n

colnames(three_grd_data$df) <- paste0("x", 1:NCOL(three_grd_data$df))

langevitour(three_grd_data$df, pointSize = 2)
```
